package info.edu.utils;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Field;
import java.util.*;

import info.edu.model.User;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/*
 * Created:20191218
 * Xml Generic Tool Service
 * */
public class XmlUtils<T> {

    // XML 存放路径
    private final String defaultPath = "src/xml/";

    /*
     * 查询全部xml数据
     * */
    public List<Map> readInnerXml(String path) throws Exception{
        File dir = new File(defaultPath+path);
        if (!dir.exists()) {
            dir.createNewFile();
            Document dom = DocumentHelper.createDocument();
            Element root = dom.addElement("table");
            String dirpath = dir+"";
            UtilsForXML.writeToXML(dom, dirpath);
        }
        String dirPath = dir+"";
        Document dom = UtilsForXML.getDocument(dirPath);
        Element root = dom.getRootElement();
        List<Map> tables = new ArrayList<>();
        List list = root.elements();
        //处理root节点数据
        for (int i = 0; i < list.size(); i++) {
            Element person = (Element) list.get(i);
            Map<String,Object> res = new HashMap<>();
            //获取子节点数据
            for (Iterator iterator = person.elementIterator(); iterator.hasNext();) {
                Element type = (Element) iterator.next();
                res.put(type.getName(),type.getStringValue());
            }
            tables.add(res);
        }
        return tables;
    }

    /*
     * 根据item的属性id查询xml
     * */
    public T readInnerXmlById(String path,String id) throws Exception{
        File dir = new File(defaultPath+path);
        String dirPath = dir+"";
        Document dom = UtilsForXML.getDocument(dirPath);
        Element root = dom.getRootElement();
        Element beQuery = (Element)root.selectSingleNode("//item[@account="+id+"]");
        Map<String,Object> res = new HashMap<>();
        if(beQuery!=null){
            for (Iterator iterator = beQuery.elementIterator(); iterator.hasNext();) {
                Element type = (Element) iterator.next();
                res.put(type.getName(),type.getStringValue());
            }
            return (T)res;
        }
        return null;
    }

    /*
     * 根据item的属性account查询xml
     * */
    public Map readInnerXmlByAccount(String path,String account) throws Exception{
        File dir = new File(defaultPath+path);
        String dirPath = dir+"";
        Document dom = UtilsForXML.getDocument(dirPath);
        Element root = dom.getRootElement();
        Element beQuery = (Element)root.selectSingleNode("//item[@account="+account+"]");
        Map<String,Object> res = new HashMap<>();
        if(beQuery!=null){
            for (Iterator iterator = beQuery.elementIterator(); iterator.hasNext();) {
                Element type = (Element) iterator.next();
                res.put(type.getName(),type.getStringValue());
            }
            return res;
        }
        return null;
    }

    /*
     * 增加xml数据
     * */
    public int writeInnerXml(String path,T list) throws Exception{
        File dir = new File(defaultPath+path);
        if (!dir.exists()) {
            dir.createNewFile();
            Document dom = DocumentHelper.createDocument();
            Element root = dom.addElement("table");
            String dirpath = dir+"";
            UtilsForXML.writeToXML(dom, dirpath);
        }
        int id = 1;
        String dirPath = dir+"";
        Document dom = UtilsForXML.getDocument(dirPath);
        Element root = dom.getRootElement();
        Field[] fields = list.getClass().getDeclaredFields();
        Element newPerson = root.addElement("item");
        for(int i=0; i<fields.length; i++){
            Field f = fields[i];
            f.setAccessible(true);
            if(f.getName() == "id" ){
                newPerson.addAttribute("id", f.get(list).toString());
            }
            if(f.getName() == "account"){
                newPerson.addAttribute("account", f.get(list).toString());
            }
            Element element = newPerson.addElement(f.getName());
            element.setText(f.get(list).toString());
        }
        UtilsForXML.writeToXML(dom, dirPath);
        return id;
    }

    /*
     * 根据item属性id修改xml数据
     * */
    public int UpdateXml(String path,String id ,T t) throws Exception{
        File dir = new File(defaultPath+path);
        String dirPath = dir+"";
        Document dom = UtilsForXML.getDocument(dirPath);
        Element root = dom.getRootElement();
        Element beQuery = (Element)root.selectSingleNode("//item[@id="+id+"]");
        Field[] fields = t.getClass().getDeclaredFields();
        for(int i=0; i<fields.length; i++){
            Field f = fields[i];
            f.setAccessible(true);
            try {
                beQuery.element(f.getName()).setText(f.get(t).toString());
            }catch (Exception s){

            }
        }
        UtilsForXML.writeToXML(dom,dirPath);
        return 1;
    }

    /*
     * 根据item属性id删除xml数据
     * */
    public int DeleteXml(String id,String path) throws Exception{
        File dir = new File(defaultPath+path);
        String dirPath = dir+"";
        Document dom = UtilsForXML.getDocument(dirPath);
        Element root = dom.getRootElement();
        Element beQuery = (Element)root.selectSingleNode("//item[@id="+id+"]");
        if(beQuery != null) {
            beQuery.getParent().remove(beQuery);
            UtilsForXML.writeToXML(dom, dirPath);
            return 0;
        }else {
            return 1;
        }
    }


    public enum XMLFILES{
        USER("user.xml","用户信息存文件"),
        ITEM("item.xml","商品文件");

        private String path;
        private String remark;
        XMLFILES(String path,String remark){
            this.path = path;
            this.remark = remark;
        }

        public String getPath(){
            return this.path;
        }
    }
}
